@@ -136,7 +136,9 @@ read_ima_dyn_dir <- function(dir, extra = NULL, verbose = FALSE) {
     if (mrs_data$meta$NumberOfTransients == Ndyns(mrs_data)) {
       return(mrs_data)
     } else if (mrs_data$meta$NumberOfTransients * 2 == Ndyns(mrs_data)) {
-      return(mrs_data)
+      out <- list(metab = get_fh_dyns(mrs_data), ref = get_sh_dyns(mrs_data))
+      class(out) <- c("list", "mrs_data")
+      return(out)
     } else {
       return(extract_dkd_wref_scans(mrs_data))
     }
